#[macro_use]
extern crate serde_derive;

use std::collections::HashMap;
use std::fs::File;
use std::io::Read;

#[derive(Serialize, Deserialize)]
/// The data contained from the json file generated by python.
pub struct JsonData {
    /// The names of the IAs.
    pub ias: Vec<String>,

    /// The results of the battles.
    pub battles: HashMap<String, f64>,
}

impl JsonData {
    /// Creates some test data.
    pub fn test() -> JsonData {

        // Open the file
        let mut file = File::open("./assets/test.json")
            .expect("Couldn't open file");

        let mut content = String::new();
        file.read_to_string(&mut content)
            .expect("Couldn't read data");

        serde_json::from_str(&content)
            .expect("Couldn't parse json")
    }
}

#[derive(Serialize, Deserialize, Clone)]
/// An IA.
pub struct Ia {
    /// The name of the ia.
    pub name: String,

    /// The number of battles won by the ia.
    pub victory_count: usize,

    /// The number of battles lost by the ia.
    pub defeat_count: usize,
}

#[derive(Serialize, Deserialize)]
pub struct Data {
    /// The IAs
    pub ias: Vec<Ia>,

    /// The battles.
    ///
    /// This hashmap is symmetric.
    pub battles: HashMap<String, f64>,

    /// The ias sorted by victory count.
    pub sorted_ias: Vec<Ia>,
}

impl Data {
    /// Creates some test data.
    pub fn test() -> Data {
        JsonData::test().into()
    }
}

impl From<JsonData> for Data {
    fn from(d: JsonData) -> Data {
        let mut ias = vec![];

        for ia in d.ias {
            ias.push(Ia {
                name: ia,
                victory_count: 0,
                defeat_count: 0,
            })
        }

        let mut battles = HashMap::new();

        for (key, val) in d.battles {

            let playing_ias = key.split("/")
                .collect::<Vec<_>>();

            let key1 = format!("{}/{}", playing_ias[0], playing_ias[1]);
            let key2 = format!("{}/{}", playing_ias[1], playing_ias[0]);

            battles.insert(key1, val);
            battles.insert(key2, val);

            if val != 0.5 {
                let (winner, loser) = if val > 0.5 {
                    (playing_ias[0], playing_ias[1])
                } else {
                    (playing_ias[1], playing_ias[0])
                };

                for ia in &mut ias {
                    if ia.name == winner {
                        ia.victory_count += 1;
                    }

                    if ia.name == loser {
                        ia.defeat_count += 1;
                    }
                }
            }
        }

        let mut sorted_ias = ias.clone();
        sorted_ias.sort_by_key(|ia| {
            (std::usize::MAX - ia.victory_count, ia.defeat_count)
        });

        Data {
            ias,
            sorted_ias,
            battles,
        }
    }
}
